// -----------------------------------------------------------------------------
// COPYRIGHT (C) 2020 - AUTHOR - MANISH JOSHI
//
// CREATED USING SCILAB-6.1.0 ON WINDOWS-10 64-bit
//
// DATE OF CREATION: 26 Oct, 2020 
//
// THIS FILE IS A PART OF LSF_Toolbox 
// -----------------------------------------------------------------------------


// THIS IS THE CALLBACK FUNCTION FOR EXPONENTIAL FIT (y = a * exp(b*x) )
function ExpBtn_callback(handles)
	[x,y] =  Data()
	n = size(x)(1)
	if n==1 then
		messagebox("Fitting is not possible please enter more data points ", "Error!!", "error");
		return;
	end
	dx = (max(x)- min(x))/5
	X = linspace(min(x)-dx,max(x)+dx,1000)
	Xnew = x 
	Ynew = log( y )
	b  = ( n * sum ( Xnew .* Ynew ) - sum ( Xnew )* sum ( Ynew ) ) /( n* sum ( Xnew .* Xnew ) - ( sum ( Xnew) ) ^2 )
	lna = sum ( Ynew ) / n - b *sum( Xnew ) / n
	a = exp( lna )
	wm = ones(n,1);
	x0 = real([a ; b]);
	tm = x; ym = y;
	function y=yth(t, x)
		y  = x(1)*exp(x(2)*t)
	endfunction
	
	function e=myfun(x, tm, ym, wm) 
		e = wm.*( yth(tm, x) - ym )
	endfunction
	if min(x)<=0 then
		xopt=x0;
	else
		xinf = [-%inf,-%inf];
		xsup = [%inf, %inf];
		[f,xopt, gopt] = leastsq(list(myfun,tm,ym,wm),x0);
	end
	Y = xopt(1)*exp(xopt(2)*X);
	YR = xopt(1)*exp(xopt(2)*x);
	Ymean = sum(YR)/n;
	SSR = sum((Ymean-YR).^2);
	SSE = sum((y-YR).^2);
	SST = SSE + SSR;
	Rsqr = SSR/SST;
	DFE = n-2;
	if n == 2 then
			ARsqr = %nan;
			RMSE = %nan;
		else
			ARsqr = 1-((1-Rsqr)*(n-1)/(n-2));
			RMSE = sqrt(SSE/(n-2))
		end
	plot(x,y,"k.",X,Y,"g","thickness",3);
	LegStr = msprintf("y = %0.4G exp(%0.4G x)",xopt(1) , xopt(2) )
	LabelGraph(handles, LegStr, 2);
	ResultStr = "======= Exponential Fitting =======\n\n"+...
	"The exponential function obtained is : \n\n" +...
	msprintf("   y = %0.4G exp(%0.4G x)",xopt(1),xopt(2))+"\n\nGoodness of fit : \n " + ....
	msprintf("  SSE = %g ", SSE)+  "\n" + msprintf("   R^2= %g ", Rsqr)+  "\n" + msprintf("   DFE = %g ", DFE)+  "\n" + ....
	msprintf("   Adj. R^2 = %g ", ARsqr)+  "\n" + msprintf("   RMSE = %g ", RMSE)+  "\n" ;
	InsertTable(handles,"Exponential Fitting", SSE,Rsqr, DFE, ARsqr, RMSE, 2)
	ResultString(handles,ResultStr)
endfunction
